## Universidad de Valparaiso
## MFA-NIPALS
## 29 Agosto 2023

library(FactoMineR) ## MFA complete data
library(missMDA) ## RIMFA
library(ade4) ## NIPALS
library(corrplot) ## correlation matrix

## MFA.nipals() ## MFA-NIPALS with NA
## fmd() ## random NA

## data

dataAFM.ok <- read.table("dataAFM.ok.txt",header=TRUE,
                         sep="",dec=".",row.names = 1)

library(corrplot)
x11()
rr=cor(dataAFM.ok)
corrplot(rr, method = 'number') # colorful number

## histogram

x11()
par(mfrow=c(3,3))
hist(dataAFM.ok[,1],col="red",main="Math",xlab="qualifications")
hist(dataAFM.ok[,2],col="red",main="Sciences",xlab="qualifications")
hist(dataAFM.ok[,3],col="red",main="Spanish",xlab="qualifications")
hist(dataAFM.ok[,4],col="green",main="Math2",xlab="qualifications")
hist(dataAFM.ok[,5],col="green",main="Sciences2",xlab="qualifications")
hist(dataAFM.ok[,6],col="green",main="Spanish2",xlab="qualifications")
hist(dataAFM.ok[,7],col="blue",main="Math3",xlab="qualifications")
hist(dataAFM.ok[,8],col="blue",main="Sciences3",xlab="qualifications")
hist(dataAFM.ok[,9],col="blue",main="Spanish3",xlab="qualifications")


## MFA complete data

mfa.complete <- MFA(dataAFM.ok,group=c(3,3,3),type=rep("s",3),
                    name.group = c("Period1", "Period2","Period3"))


## MFA-NIPALS with 10% of NA

## MFA.nipals(data,group,it)


MFA.nipals <- function(data, group, it) {
  ## group: is a vector with the number variables per group.
  ## it: is the number iterations in NIPALS.
  
  nipalsg1 <- ade4::nipals(data[,1:group[1]], niter=it)
  nipalsg2 <- ade4::nipals(data[,(group[1]+1):sum(group[1:2])], niter=it)
  nipalsg3 <- ade4::nipals(data[,(sum(group[1:2])+1):sum(group)], niter=it)
  
  dataZ.na <- cbind(data[,1:group[1]]/nipalsg1$eig[1],
                    data[,(group[1]+1):sum(group[1:2])]/nipalsg2$eig[1],
                    data[,(sum(group[1:2])+1):sum(group)]/nipalsg3$eig[1])
  
  nipals.global.na <- ade4::nipals(dataZ.na,niter=it)
  
  ## individual factor map
  plot(nipals.global.na$li[,1],nipals.global.na$li[,2],
       xlab="dim1",ylab="dim2",pch=19,
       main="Individual factor map with MFA-NIPALS")
  text(nipals.global.na$li[,1]+0.001,nipals.global.na$li[,2]+0.2,
       1:50)
  abline(h=0)
  abline(v=0)
  
  ## corcircle
  
  #### circulo
  radio <- 1
  t <- seq(0, 2*pi, length.out = 100)
  
  x <- cos(t)*radio
  y <- sin(t)*radio
  
  ### corcircle
  
  plot(nipals.global.na$co[,1],nipals.global.na$co[,2],
       xlab="dim1",ylab="dim2",pch=19,
       main="Correlation circle map with MFA-NIPALS",
       ylim=c(-1.2,1.2),xlim=c(-1.1,1.1))
  text(nipals.global.na$co[,1],nipals.global.na$co[,2]+0.001,
       colnames(data),
       col=c(rep(2,group[1]),rep(3,group[2]), rep(4,group[3])))
  lines(x, y, lwd = 1)
  abline(h=0)
  abline(v=0)
  
  #list(nipals.global.na)
  
}

dataAFM.ok.na10 <- read.table("data afm con el 10.txt",header=TRUE,
                              sep="",dec=".",row.names = 1)

xx <- colnames(dataAFM.ok)

colnames(dataAFM.ok.na10) <- xx  

MFAnipals1 <- MFA.nipals(dataAFM.ok.na10, group=c(3,3,3), it=5000)


### other example with 10% of NA

set.seed(15)

## fmd(Xo,a) ## Xo data with NA, a: percentage of NA

fmd <- function(Xo,a){
  X. <- as.matrix(Xo)
  n <- nrow(X.); p <- ncol(X.); N <- n*p
  m <- sample(N, round(a*N,0)) ; d <- length(m)
  for(j in 1:d){
    X.[m[j]] <- NA
  }
  return(X.)
}

dataAFM.ok.na10.OTHER <- fmd(dataAFM.ok,0.1)

MFAnipals1 <- MFA.nipals(dataAFM.ok.na10.OTHER, group=c(3,3,3), it=5000)



### RIMFA example

dataAFM.ok.na5 <- read.table("data afm con el 5.txt", header=TRUE,
                             sep="",dec=".",row.names = 1)

xx <- colnames(dataAFM.ok)

colnames(dataAFM.ok.na5) <- xx  


afm.impute.na5 <- imputeMFA(dataAFM.ok.na5,group=c(3,3,3))
afm.na5 <- MFA(afm.impute.na5$completeObs,group=c(3,3,3),
               type=rep("s",3),name.group = c("Period1","Period2","Period3"))










